//+------------------------------------------------------------------+
//|                                                  Time Trader.mq4 |
//|                                         Copyright 2018, AvailFX. |
//|                            https://www.mql5.com/en/users/availfx |
//+------------------------------------------------------------------+
#property copyright "Copyright 2018, AvailFX."
#property link      "https://www.mql5.com/en/users/availfx"
#property version   "1.10"
#property strict

//--- importing standard library
#import "stdlib.ex4"
string ErrorDescription(int error_code);
#import
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum OrderTypes
  {
   buy,  // Buy
   sell  // Sell
  };
input int         MagicNum=131313;     // Magic Number  
input double      Lots=0.1;
extern double     TP = 10;             // Take Profit(PIPs)
extern double     SL = 10;             // Stop Loss (PIPs)
input int         Slippage=60;
input OrderTypes  types= 0;            // Type of Order
input int         HH = 5;              // Hours
input int         MM = 10;             // Minutes
input int         SS = 50;             // Seconds  

bool firstRun=true;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
      if(HH>=24 || MM >= 60 || SS >= 60){Alert("Wrong Time Parameter.."); return(INIT_FAILED);}
//---
   if(Digits==5 || Digits==3){ SL*=10; TP*=10;}
//---
   EventSetMillisecondTimer(1);
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   EventKillTimer();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnTimer()
  {
   if(Hour()==HH && Minute()==MM && Seconds()>=SS-1)
     {
      while(OrdersOpened((ENUM_ORDER_TYPE)types)==0)
        {OpenOrder((ENUM_ORDER_TYPE)types);}
     }
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
  
//---
   if(IsTesting())
      {
         if(Hour()==HH && Minute()==MM && Seconds()>=SS-1)
           {
            while(OrdersOpened((ENUM_ORDER_TYPE)types)==0)
              {OpenOrder((ENUM_ORDER_TYPE)types);}
           }
      }
  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Check For Opened Orders                                          |
//+------------------------------------------------------------------+
int OrdersOpened(ENUM_ORDER_TYPE type=-1)
  {
   int total= OrdersTotal();
   if(total == 0) return 0;
   int cnt=0;
   for(int i=total;i>=0;i--)
     {
      if(!OrderSelect(i,SELECT_BY_POS,MODE_TRADES)) continue;
      if(OrderCloseTime()!=0) continue;
      if(OrderSymbol()!=Symbol()) continue;
      if(OrderMagicNumber()!=MagicNum) continue;
      if(type!=-1)
         if(OrderType()!=type) continue;

      cnt++;
     }
   return cnt;
  }
//+------------------------------------------------------------------+
//| Open Orders                                                      |
//+------------------------------------------------------------------+
void OpenOrder(ENUM_ORDER_TYPE type)
  {
   if(OrdersOpened(type) > 0) return;
   int ticket=-1;
   ResetLastError();
   RefreshRates();
   if(type==OP_BUY)
     {
      ticket=OrderSend(Symbol(),OP_BUY,NL(Lots),Ask,Slippage,NP(Ask-SL*Point),NP(Ask+TP*Point),"Buy Order",MagicNum);
     }
   else if(type==OP_SELL)
     {
      ticket=OrderSend(Symbol(),OP_SELL,NL(Lots),Bid,Slippage,NP(Bid+SL*Point),NP(Bid-TP*Point),"Sell Order",MagicNum);
     }

   if(ticket>0)
      Print("Order Opened Successfully. #",ticket);
   else
      Print("Failed to Open Order, Error : ",ErrorDescription(GetLastError()));
  }
//+------------------------------------------------------------------+
//| Normalize Price                                                  |
//+------------------------------------------------------------------+
double NP(double price)
  {
   double tickSize=SymbolInfoDouble(Symbol(),SYMBOL_TRADE_TICK_SIZE);
   return (MathRound(price/tickSize) * tickSize );
  }
//+------------------------------------------------------------------+
//| Normalize Lots                                                   |
//+------------------------------------------------------------------+
double NL(double lot)
  {
   double lotStep= SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_STEP);
   double minLot = SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MIN);
   double maxLot = SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MAX);

//-- Check if the lots is < min Lot size allowed or greater than the max
   if(lot<minLot) {PrintFormat("The required Lot Size(%.3f) is less than the Minimum Lot Size Allowed (%.2f)...",lot,minLot); return(minLot); }
   if(lot>maxLot) {PrintFormat("The required Lot Size(%.3f) is less than the Minimum Lot Size Allowed (%.2f)...",lot,maxLot); return(maxLot);}
   return (MathRound(lot / lotStep) * lotStep);
  }  
//+------------------------------------------------------------------+
